### source('~/scripts/opmos/routine0_prepare.R')
### for (k in 1:20){dev.off()}
### source(paste0(pdir,'/routine1_hdata.R'))

rm(list=ls())
source('~/R/util.R')
library(plyr)
library(SDMTools)
library(caret)


multistattraining=0

###########################################################################
print('********** Read arguments')
if(1==1){
    args = commandArgs(TRUE)
    what                  <- args[1]
    indir                 <- args[2]
    workdir               <- args[3]
    whichoutput           <- args[4]
    station               <- args[5]
    pollutant             <- args[6]
    d1                    <- args[7]
    d2                    <- args[8]
    experiment            <- args[9]
    criteria              <- as.numeric(args[10])
    pdir                  <- args[11] 
    domain                <- args[12]
    allstation            <- args[13]
    outputfile=paste0(station,'_',pollutant,'_',what)
}else{         
  
    what                  ="obs"
    indir                 ="/gpfs/scratch/bsc32/bsc32883/data/extractions_et"
    workdir               ="/gpfs/scratch/pr1ehu00/pr1ehu18/data/opmos//debug//opmos_b007_MAD-RUR_2015010100_2015093023_vdebug/tmp/tmp_1_2015010100_2015093023_ES1802A_obs"
    whichoutput           ="pdf-stat-hdata-dxdata-mxdata-h24xdata-dmaxxdata-dmda8xdata"
    station               ="ES1802A"
    pollutant             ="PM10"
    d1                    ="2015010100"
    d2                    ="2015093023"
    experiment            ="b007"
    criteria              =75
    pdir                  ="/home/pr1ehu00/pr1ehu18/scripts/opmos"
    domain                ="IP4"
    allstation            ="MAD-RUR"
    outputfile            ="ES1802A_PM10_obs"

    

    id=toString(system("date '+%Y%m%d%H%M%S'"))
    outputfile            =paste0(station,"_",pollutant,"_",what,"_",id)
    outputfile            =paste0(station,"_",pollutant,"_",what,"_0.95")
    outputfile            =paste0(station,"_",pollutant,"_",what)
    if(multistattraining==1){
        outputfile            =paste0(station,"_",pollutant,"_",what,"_multistattraining")
    }

    
    workdir               ="/gpfs/scratch/pr1ehu00/pr1ehu18/data/opmos/debug/interactive/"
}


print('********** Print arguments')
cat(paste0('    what                  ="',what,'"\n'))
cat(paste0('    indir                 ="',indir,'"\n'))
cat(paste0('    workdir               ="',workdir,'"\n'))
cat(paste0('    whichoutput           ="',whichoutput,'"\n'))
cat(paste0('    station               ="',station,'"\n'))
cat(paste0('    pollutant             ="',pollutant,'"\n'))
cat(paste0('    d1                    ="',d1,'"\n'))
cat(paste0('    d2                    ="',d2,'"\n'))
cat(paste0('    experiment            ="',experiment,'"\n'))
cat(paste0('    criteria              =',criteria,'\n'))
cat(paste0('    pdir                  ="',pdir,'"\n'))
cat(paste0('    domain                ="',domain,'"\n'))
cat(paste0('    allstation            ="',allstation,'"\n'))
cat(paste0('    outputfile            ="',outputfile,'"\n'))


print('********** Preliminaries')
if(pollutant=='O3'  ){limit=limit_o3}
if(pollutant=='NO2' ){limit=limit_no2}
if(pollutant=='PM10'){limit=limit_pm10}
nlimit=length(limit)-1
if(what=='obs'              ){cold='black'}      ; colo='black'
if(what=='mod'              ){cold='darkred'}    ; colm='darkred'
if(what=='kf0'              ){cold='orange'}    
if(substr(what,1,2)=='ma'   ){cold='darkorchid3'}
if(substr(what,1,3)=='kfs'  ){cold='dodgerblue1'}
if(substr(what,1,3)=='kfd'  ){cold='dodgerblue4'}
if(substr(what,1,2)=='an'   ){cold='darkgreen'}
if(substr(what,1,4)=='kfsan'){cold='blue'}
if(substr(what,1,2)=='ml'   ){cold='green'}

pcho=17 ; pchm=16 ; pchd=16
if(what=='mod'){
    colomd=c(colo,colm) ; legomd=c('obs','mod') ; pchomd=c(pcho,pchm)
    colmd=colm          ; legmd='mod'           ; pchmd=pchm
}else{
    colomd=c(colo,colm,cold) ; legomd=c('obs','mod',what) ; pchomd=c(pcho,pchm,pchd)
    colmd=c(colm,cold)       ; legmd=c('mod',what)        ; pchmd=c(pchm,pchd)
}



print('********** Define time and create dataframe')
date1=ISOdate(substr(d1,1,4),substr(d1,5,6),substr(d1,7,8),substr(d1,9,10))
date2=ISOdate(substr(d2,1,4),substr(d2,5,6),substr(d2,7,8),substr(d2,9,10))
htime <- seq(date1,date2,"hours")    ; nhtime <- length(htime)
dtime <- seq(date1,date2,"days")     ; ndtime <- length(dtime)
mtime <- seq(date1,date2,"months")   ; nmtime <- length(mtime)

#htime=strptime(htime,format='%Y-%m-%d %H:%M:%S',tz='UTC')






print('********** Get hourly obs/mod data')
res <- get_hdata(pollutant,station,htime,c('obs','mod'),domain,experiment,indir,allstation)
hobs <- res[1,]
hmod <- res[2,]
if(experiment=='b007'){
    w=which(hmod==0)
    if(length(w)!=0){hmod[w]=hmod[w+1]}  ## temporary !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
}
print(summary(hobs))
print(summary(hmod))


print('********** Remove observations > 1000 ug/m3')
w=which(hobs > 1000)
print(paste('Numlber of observations > 1000 ug/m3 :',length(w)))
if(length(w)!=0){
    hobs[w]=NaN  
    print(htime[w])
}


print('********** Check the number of available obs/mod data')
n_finiteobs  <- length(which(is.finite(hobs)))
n_finitemod  <- length(which(is.finite(hmod)))
n_nonzeroobs <- length(which(hobs!=0))
n_nonzeromod <- length(which(hmod!=0))
print(paste('n_finitemod =',n_finitemod ,100*n_finitemod /nhtime,'%'))
print(paste('n_finiteobs =',n_finiteobs ,100*n_finiteobs /nhtime,'%'))
print(paste('n_nonzeromod=',n_nonzeromod,100*n_nonzeromod/nhtime,'%'))
print(paste('n_nonzeroobs=',n_nonzeroobs,100*n_nonzeroobs/nhtime,'%'))







if(100*n_finitemod/nhtime >= criteria &  100*n_nonzeromod/nhtime >= criteria &
   100*n_finiteobs/nhtime >= criteria &  100*n_nonzeroobs/nhtime >= criteria){


    print('********** Quality check')
    lim=1e5
    if(maxok(hobs)>lim){ print(paste('BUG(hobs)=',maxok(hobs))) ;  hobs[which(hobs>lim)]=NaN }
    if(maxok(hmod)>lim){ print(paste('BUG(hmod)=',maxok(hmod))) ;  hmod[which(hmod>lim)]=NaN }

   
    flag_negativeallowed=0   ;  legneg='NoNegative'                    



    source(paste0(pdir,'/routine1_hdata.R'))

    if(is.finite(maxok(hdata))==TRUE){  

        if(what=='obs' | what=='mod'){wversion=1}else{wversion=c(1,nversions)}
        wversion=1        
        source(paste0(pdir,'/routine2_timeseriesdata.R'))
        source(paste0(pdir,'/routine3_write.R'))        
        if(what!='obs'){ for (k in wversion){source(paste0(pdir,'/routine4_plot.R'))} }       
    }

    if(0==1){
        hkfs=read.table(paste0('/esarchive/scratch/hpetetin/data/opmos/debug/opmos_b007_RUR_2015010100_2015093023_v20190107182811/data_2015010100_2015093023_',station,'/',station,'_',pollutant,'_kfsRMSE_hdata.csv'),skip=1,sep=',')[,4]
        w=which(is.finite(hdata_all[1,1,,1])==TRUE & is.finite(hmod)==TRUE)
        w=which(is.finite(hdata_all[1,1,,1])==TRUE & is.finite(hmod)==TRUE & is.finite(hkfs)==TRUE & is.finite(hobs)==TRUE)
        print(get_statistics(hobs[w],hmod[w])$cleg)
        print(paste(get_statistics(hobs[w],hmod[w])$cres,'(mod)'))
        print(paste(get_statistics(hobs[w],hkfs[w])$cres,'(kfs)'))
        print(paste(get_statistics(hobs[w],hdata_all[1,1,w,1])$cres  ,'(ml)'))
    }
}
