#!/bin/bash 
########################################################################################
#                                 TOP_OPMOS (Operational_like MOS)
########################################################################################
#
# This script handles all types of MOS (model output statistics) to correct the model outputs 
# based on the observations
#
######### STEPS :
#     0) cancel currently running jobs and preliminaries
#     1) generate the joblist
#     2) launch opmos jobs
#     3) wait the end of the jobs
#     4) save usefull files
#     5) start postprocessing
#
######### IMPORTANT USER-DEFINED PARAMETERS :
#  - experiment       : experiment among CALIOPE/b007/CAMS50
#  - d1, d2           : dates (YYYYMMDDHH with HH from 00 to 23)
#  - station          : one station name or the code for a group of stations
#  - pollutant        : one pollutant or a list of list of pollutants
#  - what             : one MOS model or a list of MOS model (always include "obs" and "mod")
#  - jobtime          : maximum time (in hours) of the job (one job corresponds to all <what> MOS models on 1 station for 1 pollutant
#  - criteria         : minimum percentage of observational data required for taking into account a station
#  - nmaxrunningjobs  : maximum number of jobs running in parallel (recommanded : 800 on P9 or 1800 on MN4)
#  - outdirtmp        : general output directory (where a folder will be created)
#  - domain           : only used for CALIOPE
#  - whichoutput      : which output files you want to keep, could be any combination of pdf/stat/hdata/h24data/...
#
########################################################################################

nmosperjob=1

whichoutput='pdf-stat-hdata-dxdata-mxdata-h24xdata-dmaxxdata-dmda8xdata'

for k in 1 ; do 
    case $k in 
	1) experiment=b007   ; d1=2015010100 ; d2=2015093023 ; station='MAD-RUR' ; pollutant='O3' ;;
	2) experiment=b007   ; d1=2015010100 ; d2=2015093023 ; station='MAD-SUB' ; pollutant='O3 PM10 NO2' ;;
	3) experiment=b007   ; d1=2015010100 ; d2=2015093023 ; station='MAD-URB' ; pollutant='O3 PM10 NO2' ;;
	4) experiment=cams50 ; d1=2016010100 ; d2=2017123123 ; station='MAD-RUR' ; pollutant='O3' ;;
	5) experiment=cams50 ; d1=2016010100 ; d2=2017123123 ; station='MAD-SUB' ; pollutant='O3' ;;	
	6) experiment=cams50 ; d1=2016010100 ; d2=2017123123 ; station='MAD-URB' ; pollutant='O3' ;;

	10) experiment=b007   ; d1=2015010100 ; d2=2015093023 ; station='IP'     ; pollutant='O3 PM10 NO2' ;;
	11) experiment=b007   ; d1=2015010100 ; d2=2015093023 ; station='IP-RUR' ; pollutant='O3 PM10 NO2' ;;
	12) experiment=b007   ; d1=2015010100 ; d2=2015093023 ; station='IP-SUB' ; pollutant='O3 PM10 NO2' ;;
	13) experiment=b007   ; d1=2015010100 ; d2=2015093023 ; station='IP-URB' ; pollutant='O3 PM10 NO2' ;;
	14) experiment=cams50 ; d1=2016010100 ; d2=2017123123 ; station='RUR' ; pollutant='O3' ;;
	15) experiment=cams50 ; d1=2016010100 ; d2=2017123123 ; station='SUB' ; pollutant='O3' ;;	
	16) experiment=cams50 ; d1=2016010100 ; d2=2017123123 ; station='URB' ; pollutant='O3' ;;

	21) experiment=b007   ; d1=2015010100 ; d2=2015093023 ; station='RUR' ; pollutant='PM10' ;;
	22) experiment=b007   ; d1=2015010100 ; d2=2015093023 ; station='SUB' ; pollutant='PM10' ;;
	23) experiment=b007   ; d1=2015010100 ; d2=2015093023 ; station='URB' ; pollutant='PM10' ;;
	24) experiment=cams50 ; d1=2016010100 ; d2=2017123123 ; station='RUR' ; pollutant='O3' ;;
	25) experiment=cams50 ; d1=2016010100 ; d2=2017123123 ; station='SUB' ; pollutant='O3' ;;	
	26) experiment=cams50 ; d1=2016010100 ; d2=2017123123 ; station='URB' ; pollutant='O3' ;;
	*) exit 0 ;;
    esac

    case $experiment in 
#	b007) what='obs mod ma1 ma2 ma3 ma4 ma5 ma6 ma7 ma10 ma15 ma20 kfsRMSE kfdRMSE an-C-0 an-C-1 an-C-1_H-1 an-C-1_T-1 an-C-1_T-1_H-1 an-C-1_P-1 an-C-1_P-1_H-1 an-C-1_P-1_T-1 an-C-1_P-1_T-1_H-1 an-C-1_D-1 an-C-1_D-1_H-1 an-C-1_D-1_T-1 an-C-1_D-1_T-1_H-1 an-C-1_D-1_P-1 an-C-1_D-1_P-1_H-1 an-C-1_D-1_P-1_T-1 an-C-1_D-1_P-1_T-1_H-1 an-C-1_S-1 an-C-1_S-1_H-1 an-C-1_S-1_T-1 an-C-1_S-1_T-1_H-1 an-C-1_S-1_P-1 an-C-1_S-1_P-1_H-1 an-C-1_S-1_P-1_T-1 an-C-1_S-1_P-1_T-1_H-1 an-C-1_S-1_D-1 an-C-1_S-1_D-1_H-1 an-C-1_S-1_D-1_T-1 an-C-1_S-1_D-1_T-1_H-1 an-C-1_S-1_D-1_P-1 an-C-1_S-1_D-1_P-1_H-1 an-C-1_S-1_D-1_P-1_T-1 an-C-1_S-1_D-1_P-1_T-1_H-1 kfsan-C-0 kfsan-C-1 kfsan-C-1_H-1 kfsan-C-1_T-1 kfsan-C-1_T-1_H-1 kfsan-C-1_P-1 kfsan-C-1_P-1_H-1 kfsan-C-1_P-1_T-1 kfsan-C-1_P-1_T-1_H-1 kfsan-C-1_D-1 kfsan-C-1_D-1_H-1 kfsan-C-1_D-1_T-1 kfsan-C-1_D-1_T-1_H-1 kfsan-C-1_D-1_P-1 kfsan-C-1_D-1_P-1_H-1 kfsan-C-1_D-1_P-1_T-1 kfsan-C-1_D-1_P-1_T-1_H-1 kfsan-C-1_S-1 kfsan-C-1_S-1_H-1 kfsan-C-1_S-1_T-1 kfsan-C-1_S-1_T-1_H-1 kfsan-C-1_S-1_P-1 kfsan-C-1_S-1_P-1_H-1 kfsan-C-1_S-1_P-1_T-1 kfsan-C-1_S-1_P-1_T-1_H-1 kfsan-C-1_S-1_D-1 kfsan-C-1_S-1_D-1_H-1 kfsan-C-1_S-1_D-1_T-1 kfsan-C-1_S-1_D-1_T-1_H-1 kfsan-C-1_S-1_D-1_P-1 kfsan-C-1_S-1_D-1_P-1_H-1 kfsan-C-1_S-1_D-1_P-1_T-1 kfsan-C-1_S-1_D-1_P-1_T-1_H-1' ;;
	b007) what='obs mod ma1 ma2 ma3 ma4 ma5 ma6 ma7 ma10 ma15 ma20 kfsRMSE kfdRMSE an-C-0 an-C-0_S-0_D-0_T-0 kfsan-C-0 kfsan-C-0 kfsan-C-0_S-0_D-0_T-0 an-C-1 an-C-1_S-1_D-1_T-1 kfsan-C-0 kfsan-C-1 kfsan-C-1_S-1_D-1_T-1 an-C-2 an-C-2_S-2_D-2_T-2 kfsan-C-0 kfsan-C-2 kfsan-C-2_S-2_D-2_T-2' ;;
#	b007) what='obs mod ma1 ma2 ma3 ma4 ma5 ma6 ma7 ma10 ma15 ma20 kfsRMSE kfdRMSE an-C-0 an-C-1 an-C-1_H-1 an-C-1_T-1 an-C-1_T-1_H-1 an-C-1_P-1 an-C-1_P-1_H-1 an-C-1_P-1_T-1 an-C-1_P-1_T-1_H-1 an-C-1_D-1 an-C-1_D-1_H-1 an-C-1_D-1_T-1 an-C-1_D-1_T-1_H-1 an-C-1_D-1_P-1 an-C-1_D-1_P-1_H-1 an-C-1_D-1_P-1_T-1 an-C-1_D-1_P-1_T-1_H-1 an-C-1_S-1 an-C-1_S-1_H-1 an-C-1_S-1_T-1 an-C-1_S-1_T-1_H-1 an-C-1_S-1_P-1 an-C-1_S-1_P-1_H-1 an-C-1_S-1_P-1_T-1 an-C-1_S-1_P-1_T-1_H-1 an-C-1_S-1_D-1 an-C-1_S-1_D-1_H-1 an-C-1_S-1_D-1_T-1 an-C-1_S-1_D-1_T-1_H-1 an-C-1_S-1_D-1_P-1 an-C-1_S-1_D-1_P-1_H-1 an-C-1_S-1_D-1_P-1_T-1 an-C-1_S-1_D-1_P-1_T-1_H-1 kfsan-C-0 kfsan-C-1 kfsan-C-1_H-1 kfsan-C-1_T-1 kfsan-C-1_T-1_H-1 kfsan-C-1_P-1 kfsan-C-1_P-1_H-1 kfsan-C-1_P-1_T-1 kfsan-C-1_P-1_T-1_H-1 kfsan-C-1_D-1 kfsan-C-1_D-1_H-1 kfsan-C-1_D-1_T-1 kfsan-C-1_D-1_T-1_H-1 kfsan-C-1_D-1_P-1 kfsan-C-1_D-1_P-1_H-1 kfsan-C-1_D-1_P-1_T-1 kfsan-C-1_D-1_P-1_T-1_H-1 kfsan-C-1_S-1 kfsan-C-1_S-1_H-1 kfsan-C-1_S-1_T-1 kfsan-C-1_S-1_T-1_H-1 kfsan-C-1_S-1_P-1 kfsan-C-1_S-1_P-1_H-1 kfsan-C-1_S-1_P-1_T-1 kfsan-C-1_S-1_P-1_T-1_H-1 kfsan-C-1_S-1_D-1 kfsan-C-1_S-1_D-1_H-1 kfsan-C-1_S-1_D-1_T-1 kfsan-C-1_S-1_D-1_T-1_H-1 kfsan-C-1_S-1_D-1_P-1 kfsan-C-1_S-1_D-1_P-1_H-1 kfsan-C-1_S-1_D-1_P-1_T-1 kfsan-C-1_S-1_D-1_P-1_T-1_H-1' ;;
	cams50) what='obs mod ma1 ma2 ma3 ma7 ma10 ma15 ma20 kfsRMSE kfdRMSE an-C-0 an-C-1 kfsan-C-0 kfsan-C-1' ;;
	*) exit 0 ;;
    esac
    what='obs mod ma7 kfsRMSE ml_knn-200-30-0-11-4 ml_gbm-200-30-0-11-4-0.50'
    what='obs mod kfsRMSE ml_gbm-100-100-0-1-1-0.50 ml_gbm-100-100-0-2-1-0.50 ml_gbm-100-100-0-11-1-0.50 ml_gbm-100-100-1-1-1-0.50 ml_gbm-100-100-1-2-1-0.50 ml_gbm-100-100-1-11-1-0.50'
    what='obs mod ma1 ma2 ma3 ma4 ma5 ma6 ma7 ma15 kfsRMSE kfdRMSE an-C-0 an-C-0_S-0_D-0_T-0 kfsan-C-0 kfsan-C-0 kfsan-C-0_S-0_D-0_T-0 an-C-1 an-C-1_S-1_D-1_T-1 kfsan-C-0 kfsan-C-1 kfsan-C-1_S-1_D-1_T-1 an-C-2 an-C-2_S-2_D-2_T-2 kfsan-C-0 kfsan-C-2 kfsan-C-2_S-2_D-2_T-2 ml_gbm-100-100-0-1-1-0.50 ml_gbm-100-100-0-2-1-0.50 ml_gbm-100-100-0-11-1-0.50 ml_gbm-100-100-1-1-1-0.50 ml_gbm-100-100-1-2-1-0.50 ml_gbm-100-100-1-11-1-0.50'
    #what='obs mod ma1 ma2 ma3 ma4 ma5 ma6 ma7 ma10 ma15 ma20 kfsRMSE kfdRMSE an-C-0 an-C-1 an-C-1_H-1 an-C-1_T-1 an-C-1_T-1_H-1 an-C-1_P-1 an-C-1_P-1_H-1 an-C-1_P-1_T-1 an-C-1_P-1_T-1_H-1 an-C-1_D-1 an-C-1_D-1_H-1 an-C-1_D-1_T-1 an-C-1_D-1_T-1_H-1 an-C-1_D-1_P-1 an-C-1_D-1_P-1_H-1 an-C-1_D-1_P-1_T-1 an-C-1_D-1_P-1_T-1_H-1 an-C-1_S-1 an-C-1_S-1_H-1 an-C-1_S-1_T-1 an-C-1_S-1_T-1_H-1 an-C-1_S-1_P-1 an-C-1_S-1_P-1_H-1 an-C-1_S-1_P-1_T-1 an-C-1_S-1_P-1_T-1_H-1 an-C-1_S-1_D-1 an-C-1_S-1_D-1_H-1 an-C-1_S-1_D-1_T-1 an-C-1_S-1_D-1_T-1_H-1 an-C-1_S-1_D-1_P-1 an-C-1_S-1_D-1_P-1_H-1 an-C-1_S-1_D-1_P-1_T-1 an-C-1_S-1_D-1_P-1_T-1_H-1 kfsan-C-0 kfsan-C-1 kfsan-C-1_H-1 kfsan-C-1_T-1 kfsan-C-1_T-1_H-1 kfsan-C-1_P-1 kfsan-C-1_P-1_H-1 kfsan-C-1_P-1_T-1 kfsan-C-1_P-1_T-1_H-1 kfsan-C-1_D-1 kfsan-C-1_D-1_H-1 kfsan-C-1_D-1_T-1 kfsan-C-1_D-1_T-1_H-1 kfsan-C-1_D-1_P-1 kfsan-C-1_D-1_P-1_H-1 kfsan-C-1_D-1_P-1_T-1 kfsan-C-1_D-1_P-1_T-1_H-1 kfsan-C-1_S-1 kfsan-C-1_S-1_H-1 kfsan-C-1_S-1_T-1 kfsan-C-1_S-1_T-1_H-1 kfsan-C-1_S-1_P-1 kfsan-C-1_S-1_P-1_H-1 kfsan-C-1_S-1_P-1_T-1 kfsan-C-1_S-1_P-1_T-1_H-1 kfsan-C-1_S-1_D-1 kfsan-C-1_S-1_D-1_H-1 kfsan-C-1_S-1_D-1_T-1 kfsan-C-1_S-1_D-1_T-1_H-1 kfsan-C-1_S-1_D-1_P-1 kfsan-C-1_S-1_D-1_P-1_H-1 kfsan-C-1_S-1_D-1_P-1_T-1 kfsan-C-1_S-1_D-1_P-1_T-1_H-1' 
    #what='obs mod ma7 kfsRMSE ml_lm1-60-60 ml_gbm1-60-60-0.50 ml_gbm1-60-60-0.75 ml_gbm1-60-60-1.00'
    #what='obs mod ma1 ma2 ma3 ma7 kfsRMSE ml_lm1-60-30 ml_lm1-60-40 ml_lm1-60-50 ml_lm1-60-60 ml_gbm1-60-30-0.50 ml_gbm1-60-40-0.50 ml_gbm1-60-50-0.50 ml_gbm1-60-60-0.50 ml_gbm1-60-30-0.75 ml_gbm1-60-40-0.75 ml_gbm1-60-50-0.75 ml_gbm1-60-60-0.75 ml_gbm1-60-30-1.00 ml_gbm1-60-40-1.00 ml_gbm1-60-50-1.00 ml_gbm1-60-60-1.00'
    #what='obs mod ma1 ma2 ma3 ma7 kfsRMSE ml_lm1-60-30 ml_lm1-60-40 ml_lm1-60-50 ml_lm1-60-60 ml_gbm1-60-30-0.75 ml_gbm1-60-40-0.75 ml_gbm1-60-50-0.75 ml_gbm1-60-60-0.75 ml_lm2-60-30 ml_lm2-60-40 ml_lm2-60-50 ml_lm2-60-60 ml_gbm2-60-30-0.75 ml_gbm2-60-40-0.75 ml_gbm2-60-50-0.75 ml_gbm2-60-60-0.75 ml_lm3-60-30 ml_lm3-60-40 ml_lm3-60-50 ml_lm3-60-60 ml_gbm3-60-30-0.75 ml_gbm3-60-40-0.75 ml_gbm3-60-50-0.75 ml_gbm3-60-60-0.75'
    #what='obs mod ma1 ma2 ma3 ma7 kfsRMSE ml_lm1-60-30 ml_lm1-60-40 ml_lm1-60-50 ml_lm1-60-60 ml_gbm1-30-30-1.00 ml_gbm1-30-10-1.0 ml_gbm1-30-30-0.75 ml_gbm1-30-10-0.75 ml_gbm1b-30-30-1.00 ml_gbm1b-30-10-1.0 ml_gbm1b-30-30-0.75 ml_gbm1b-30-10-0.75 ml_gbm2-30-30-1.00 ml_gbm2-30-10-1.0 ml_gbm2-30-30-0.75 ml_gbm2-30-10-0.75 ml_gbm2b-30-30-1.00 ml_gbm2b-30-10-1.0 ml_gbm2b-30-30-0.75 ml_gbm2b-30-10-0.75'
    what='obs mod ma1 ma2 ma3 ma4 ma5 ma6 ma7 ma15 kfsRMSE kfdRMSE an-C-0 an-C-0_S-0_D-0_T-0 kfsan-C-0 kfsan-C-0 kfsan-C-0_S-0_D-0_T-0 an-C-1 an-C-1_S-1_D-1_T-1 kfsan-C-0 kfsan-C-1 kfsan-C-1_S-1_D-1_T-1 an-C-2 an-C-2_S-2_D-2_T-2 kfsan-C-0 kfsan-C-2 kfsan-C-2_S-2_D-2_T-2'
    what='obs mod ma7 ma15 kfsRMSE ml_gbm-100-100-0-1-1-0.50'
    what='obs mod ma7 kfsRMSE'

    what='obs mod ma1 ma2 ma3 ma4 ma5 ma6 ma7 ma15 kfsRMSE kfdRMSE an-C-0 an-C-0_S-0_D-0_T-0 kfsan-C-0 kfsan-C-0 kfsan-C-0_S-0_D-0_T-0 an-C-1 an-C-1_S-1_D-1_T-1 kfsan-C-0 kfsan-C-1 kfsan-C-1_S-1_D-1_T-1 an-C-2 an-C-2_S-2_D-2_T-2 kfsan-C-0 kfsan-C-2 kfsan-C-2_S-2_D-2_T-2 ml_gbm-100-100-0-1-1-0.50 ml_gbm-100-100-0-2-1-0.50 ml_gbm-100-100-0-11-1-0.50 ml_gbm-100-100-1-1-1-0.50 ml_gbm-100-100-1-2-1-0.50 ml_gbm-100-100-1-11-1-0.50'

    what='obs mod 
ma1 ma2 ma3 ma4 ma5 ma6 ma7 ma15
kfsRMSE kfdRMSE 
an-C-0 an-C-0_D-0 an-C-0_T-0 an-C-0_S-0 an-C-0_S-0_T-0 an-C-0_S-0_D-0_T-0 an-C-0_S-0_D-0_P-0_T-0_H-0
an-C-1 an-C-1_D-1 an-C-1_T-1 an-C-1_S-1 an-C-1_S-1_T-1 an-C-1_S-1_D-1_T-1 an-C-1_S-1_D-1_P-1_T-1_H-1
an-C-2 an-C-2_D-2 an-C-2_T-2 an-C-2_S-2 an-C-2_S-2_T-2 an-C-2_S-2_D-2_T-2 an-C-2_S-2_D-2_P-2_T-2_H-2
kfsan-C-0 kfsan-C-0_D-0 kfsan-C-0_T-0 kfsan-C-0_S-0 kfsan-C-0_S-0_T-0 kfsan-C-0_S-0_D-0_T-0 kfsan-C-0_S-0_D-0_P-0_T-0_H-0
kfsan-C-1 kfsan-C-1_D-1 kfsan-C-1_T-1 kfsan-C-1_S-1 kfsan-C-1_S-1_T-1 kfsan-C-1_S-1_D-1_T-1 kfsan-C-1_S-1_D-1_P-1_T-1_H-1
kfsan-C-2 kfsan-C-2_D-2 kfsan-C-2_T-2 kfsan-C-2_S-2 kfsan-C-2_S-2_T-2 kfsan-C-2_S-2_D-2_T-2 kfsan-C-2_S-2_D-2_P-2_T-2_H-2
ml_gbm-90-30-0-0-1-0.50 ml_gbm-90-30-0-1-1-0.50 ml_gbm-90-30-0-2-1-0.50 ml_gbm-90-30-0-3-1-0.50 ml_gbm-90-30-0-4-1-0.50 ml_gbm-90-30-0-6-1-0.50 ml_gbm-90-30-0-7-1-0.50 ml_gbm-90-30-0-11-1-0.50 
ml_gbm-90-30-0-0-1-0.75 ml_gbm-90-30-0-1-1-0.75 ml_gbm-90-30-0-2-1-0.75 ml_gbm-90-30-0-3-1-0.75 ml_gbm-90-30-0-4-1-0.75 ml_gbm-90-30-0-6-1-0.75 ml_gbm-90-30-0-7-1-0.75 ml_gbm-90-30-0-11-1-0.75 
ml_gbm-90-30-1-0-1-0.50 ml_gbm-90-30-1-1-1-0.50 ml_gbm-90-30-1-2-1-0.50 ml_gbm-90-30-1-3-1-0.50 ml_gbm-90-30-1-4-1-0.50 ml_gbm-90-30-1-6-1-0.50 ml_gbm-90-30-1-7-1-0.50 ml_gbm-90-30-1-11-1-0.50 
ml_gbm-90-30-1-0-1-0.75 ml_gbm-90-30-1-1-1-0.75 ml_gbm-90-30-1-2-1-0.75 ml_gbm-90-30-1-3-1-0.75 ml_gbm-90-30-1-4-1-0.75 ml_gbm-90-30-1-6-1-0.75 ml_gbm-90-30-1-7-1-0.75 ml_gbm-90-30-1-11-1-0.75'







    id=1      
    jobtime=3
    debug=1

    case $BSC_MACHINE in 
	'power')	
	    outdirtmp=/esarchive/scratch/hpetetin/data/opmos/
	    filestationsdir=/esarchive/scratch/hpetetin/data/stations ;;
	'mn4')
	    outdirtmp=/gpfs/scratch/pr1ehu00/pr1ehu18/data/opmos/
	    filestationsdir=/gpfs/scratch/pr1ehu00/pr1ehu18/data/stations ;;
    esac
    case $experiment in 
	'caliope') indir=/gpfs/scratch/bsc32/bsc32883/data/caliope ;;
	*)         indir=/gpfs/scratch/bsc32/bsc32883/data/extractions_et ;;
    esac
    if [ $debug -eq 1 ] ; then outdirtmp=$outdirtmp/debug/ ; fi
    outdir=$outdirtmp/opmos_${experiment}_${station}_${d1}_${d2}_v`date '+%Y%m%d%H%M%S'`
    #outdir=$outdirtmp/opmos_${experiment}_${station}_${d1}_${d2}_vdebug ; rm -rf $outdir ; mkdir $outdir

    filestations=$filestationsdir/stations.csv
    pdir=`pwd`
    domain=IP4
    joblist=$pdir/tmp_joblist
    criteria=75
    case $BSC_MACHINE in 
	'power') nmaxrunningjobs=800  ;;
	'mn4')   nmaxrunningjobs=1800 ;;
    esac

    #############################################################################################

    echo -e "\e[00;31m | ==============================================\e[00m"
    echo -e "\e[00;31m | ==================  TOP_MOS  =================\e[00m"
    echo -e "\e[00;31m | ==============================================\e[00m"
    echo -e "\e[00;31m |          > experiment : \e[00m $experiment"
    echo -e "\e[00;31m |          > d1         : \e[00m $d1"
    echo -e "\e[00;31m |          > d2         : \e[00m $d2"
    echo -e "\e[00;31m |          > station    : \e[00m $station"
    echo -e "\e[00;31m |          > pollutant  : \e[00m $pollutant"
    echo -e "\e[00;31m |          > nmosperjob : \e[00m $nmosperjob"
    begin=`date +%s` #(start chrono)

    ############ STEP0 : preliminaries
    echo -e "\e[00;31m | Cancel current jobs \e[00m `date`"
    for i in $(squeue | grep jopmos$id | awk '{print $1}') ; do scancel $i ; done
    for i in $(squeue | grep post$id   | awk '{print $1}') ; do scancel $i ; done

    echo -e "\e[00;31m | Preliminaries \e[00m `date`"
    username=`whoami`
    find $pdir -maxdepth 1 -name 'slurm-*out' -delete  #(in case too high number of slurm files to delete)
    rm -f $pdir/tmp*
    if [ ! -d $outdir     ] ; then mkdir -p $outdir     ; fi
    function convertsec(){     # (function for converting time in seconds, used for chrono routines)
	echo $(($1 / 86400))':'`date +%H:%M:%S -d "0000-01-01 $(($1 % 86400)) seconds"`
    }
    mkdir $outdir/other/
    mkdir $outdir/other/slurm_opmos
    mkdir $outdir/other/slurm_post


    ############ STEP1 : generate the joblist or (or use an already existing joblist)
    rm -f $joblist 
    if [ ! -f $joblist ] ; then
	echo -e "\e[00;31m | Preparing jobs list \e[00m ($joblist) `date`"
	rm -f $pdir/log_station_data
	ijob=1
	for ipollutant in $pollutant ; do
	    for istation in $station ; do

		## get the list with all stations (if not already available in filestation directory)
		filestationslist=$filestationsdir/stations_$istation
		if [ ! -f $filestationslist ] ; then
		    case $istation in
			'IP')      awk -F',' '{if(NR>1                                ){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'IP-URB')  awk -F',' '{if(NR>1 && $6=="URB"                   ){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'IP-SUB')  awk -F',' '{if(NR>1 && $6=="SUB"                   ){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'IP-RUR')  awk -F',' '{if(NR>1 && $6=="RUR"                   ){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'BCN')     awk -F',' '{if(NR>1              && $8=="Barcelona"){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'BCN-URB') awk -F',' '{if(NR>1 && $6=="URB" && $8=="Barcelona"){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'BCN-SUB') awk -F',' '{if(NR>1 && $6=="SUB" && $8=="Barcelona"){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'BCN-RUR') awk -F',' '{if(NR>1 && $6=="RUR" && $8=="Barcelona"){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'MAD')     awk -F',' '{if(NR>1              && $8=="Madrid"   ){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'MAD-URB') awk -F',' '{if(NR>1 && $6=="URB" && $8=="Madrid"   ){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'MAD-SUB') awk -F',' '{if(NR>1 && $6=="SUB" && $8=="Madrid"   ){print $1}}' $filestations > $pdir/tmp_filestations ;;
			'MAD-RUR') awk -F',' '{if(NR>1 && $6=="RUR" && $8=="Madrid"   ){print $1}}' $filestations > $pdir/tmp_filestations ;;
			*) echo $istation > $pdir/tmp_filestations ;;
		    esac     
		    cp $pdir/tmp_filestations $filestationslist
		else
		    cp $filestationslist $pdir/tmp_filestations
		fi
		if [ "$experiment" == "caliope" ] ; then
		    ## [caliope] select stations with at least one data
		    mv $pdir/tmp_filestations $pdir/tmp_filestations_all
		    ./select_stations.sh $ipollutant $d1 $d2 $pdir/tmp_filestations_all $pdir/tmp_filestations $criteria $experiment $indir
		    nstation=`   wc -l $pdir/tmp_filestations     | awk '{print $1}'` 
		    nstationall=`wc -l $pdir/tmp_filestations_all | awk '{print $1}'` 
		    echo -e "\e[00;31m |       \e[00m $istation : nstation=$nstation with $ipollutant observations (total=$nstationall stations)"	
		else
		    ## [other experiment] select stations with at least <criteria(%)> data
		    mv $pdir/tmp_filestations $pdir/tmp_filestations_all
		    Rscript $pdir/select_stations.R $ipollutant $d1 $d2 $pdir/tmp_filestations_all $pdir/tmp_filestations $criteria $experiment $indir $station > $outdir/other/log_select_stations_$ipollutant
		    #cat $outdir/other/log_select_stations_$ipollutant
		    nstation=`wc -l $pdir/tmp_filestations | awk '{print $1}'` 
		    nstationall=`wc -l $pdir/tmp_filestations_all | awk '{print $1}'` 
		    echo -e "\e[00;31m |       \e[00m $istation : nstation=$nstation with ${criteria}% of $ipollutant observations (total=$nstationall stations)"
		fi

		## loop on stations - assign model to the different jobs in function of <nmosperjob>
		while read -r istat ; do			    
		    i=0
		    ilistwhat=""
		    for iwhat in $what ; do
			if [ $i -lt $nmosperjob ] ; then
			    if [ "${ilistwhat}" == "" ] ; then
				ilistwhat="${iwhat}"
			    else				
				ilistwhat="${ilistwhat}|${iwhat}"
			    fi
			    i=$((i+1))
			fi
			if [ $i -eq $nmosperjob ] ; then
			    echo $ijob $ipollutant $istat $d1 $d2 $domain $ilistwhat >> $joblist 
			    ijob=$(($ijob+1))
			    i=0
			    ilistwhat=""
			fi
		    done
		    if [ "${ilistwhat}" != "" ] ; then
			echo $ijob $ipollutant $istat $d1 $d2 $domain $ilistwhat >> $joblist 
			ijob=$(($ijob+1))
		    fi
		
		done < $pdir/tmp_filestations
	    done
	done    
    else 
	echo -e "\e[00;31m | Using existant jobs list \e[00m ($joblist)"
    fi
    njobs=`wc -l $joblist | awk '{print $1}'`
    echo -e "\e[00;31m |   \e[00m njobs=$njobs  "


    ############ STEP2 : Launch opmos jobs
    echo -e "\e[00;31m | Start loop on jobs list \e[00m `date`"
    time_begin=`date +%s`
    for ijob in `seq 1 $njobs` ; do

	## monitor the jobs (only each 10 job to speed-up the loop (the squeue command is relatively slow, especially on MN4)
	if [ $(($ijob%10)) -eq 1 ] ; then
	    nrunningjobs=`squeue | grep $username | wc -l`
	    nerr=`   grep Err    $pdir/sl* 2> /dev/null | wc -l`
	    nkilled=`grep Killed $pdir/sl* 2> /dev/null | wc -l`
	    ncancel=`grep CANCEL $pdir/sl* 2> /dev/null | wc -l`
	    echo "$ijob/$njobs ($nrunningjobs running, max=$nmaxrunningjobs)(NERRORS/NKILLED/NCANCEL=$nerr/$nkilled/$ncancel)"
	    if [[ $nerr -gt 0 || $nkilled -gt 0 || $ncancel -gt 0 ]] ; then 
		for i in $(squeue | grep jopmos$id | awk '{print $1}') ; do scancel $i ; done
		echo "(NERRORS/NKILLED/NCANCEL=$nerr/$nkilled/$ncancel)"
		exit 0
	    fi
	else	 
	    echo "$ijob/$njobs (~$nrunningjobs running, max=$nmaxrunningjobs)"
	fi

	## launch a new job from the list only if the number of jobs currently running is lower than <nmaxrunningjobs>, otherwise wait
	ijoblaunched=0
	while [ $ijoblaunched -eq 0 ] ; do
	    if [ $nrunningjobs -lt $nmaxrunningjobs ] ; then 	 
		iwhat=`awk -v job=$ijob '{if(NR==job){print $2}}' $joblist` 	
		sbatch --time=${jobtime}:00:00 -n1 -c5 --job-name="jopmos$id" $pdir/int_opmos.sh $ijob $joblist $pdir $indir $outdir $whichoutput $experiment $criteria $station "$what" #> /dev/null 2>&1
		ijoblaunched=1
	    else
		nerr=`   grep Err    $pdir/sl* 2> /dev/null | wc -l`
		nkilled=`grep Killed $pdir/sl* 2> /dev/null | wc -l`
		ncancel=`grep CANCEL $pdir/sl* 2> /dev/null | wc -l`
		nrunningjobs=`squeue | grep $username | wc -l`
		echo "wait for launching next job... $ijob/$njobs ($nrunningjobs running, max=$nmaxrunningjobs)(NERRORS/NKILLED/NCANCEL=$nerr/$nkilled/$ncancel)"
		if [[ $nerr -gt 0 || $nkilled -gt 0 || $ncancel -gt 0 ]] ; then 
		    for i in $(squeue | grep jopmos$id | awk '{print $1}') ; do scancel $i ; done
		    echo "(NERRORS/NKILLED/NCANCEL=$nerr/$nkilled/$ncancel)"
		    exit 0
		fi		
		sleep 10
	    fi 
	done    
    done
    time_end=`date +%s`
    duration=`convertsec $((${time_end}-${time_begin}))`  
    echo -e "\e[00;31m |     \e[00m  $njobs launched in $duration)"


   
    ############ STEP3 : Wait the end of the jobs
    ## the post processing need all files to be generated
    echo -e "\e[00;31m | Wait the end of the jobs... \e[00m `date`"
    ndone=0
    while [ $ndone -lt $njobs ] ; do
	nerr=`   grep Err           $outdir/other/slurm_opmos/subslurm_* 2> /dev/null | wc -l`
	nkilled=`grep Killed        $outdir/other/slurm_opmos/subslurm_* 2> /dev/null | wc -l`
	ncancel=`grep CANCEL        $outdir/other/slurm_opmos/subslurm_* 2> /dev/null | wc -l`
	ndone=`  grep _THEEND_ $pdir/sl*         2> /dev/null | wc -l`
	nslurm=`ls $pdir/sl* 2> /dev/null | wc -l`
	sleep 10
	echo "waiting... ($ndone/$njobs)($nslurm slurm files)  (NERRORS/NKILLED/NCANCEL=$nerr/$nkilled/$ncancel) `date`"
	if [[ $nerr -gt 0 || $nkilled -gt 0 || $ncancel -gt 0 ]] ; then 
	    for i in $(squeue | grep jopmos$id | awk '{print $1}') ; do scancel $i ; done
	    echo "(NERRORS/NKILLED/NCANCEL=$nerr/$nkilled/$ncancel)"
	    exit 0
	fi
    done


    ############ STEP4 : Save (potentially) usefull files
    mv $pdir/slurm*.out $outdir/other/slurm_opmos
    mv $joblist                   $outdir/other/	
    #mv $pdir/log_station_data     $outdir/other/
    mv $pdir/tmp_filestations     $outdir/other/
    mv $pdir/tmp_filestations_all $outdir/other/
    cat $pdir/top_opmos.sh        > $outdir/other/copy_top_opmos.sh
    rm -r $outdir/tmp

    ############ STEP5 : Start post processing
    echo -e "\e[00;31m | Launch post.. \e[00m `date`"
    $pdir/top_post.sh $debug "$what" "$pollutant" $id $outdir $d1 $d2 &> $pdir/tmplog_top_post &

    echo -e "\e[00;31m | Waiting end of top_post.sh... \e[00m `date`"
    ok=0
    while [ $ok -ne 1 ] ; do
	ok=`grep END_top_post $pdir/tmplog_top_post 2> /dev/null | wc -l` 
	sleep 10
	echo "waiting ($ok)... `date`"
    done

    end=`date +%s`
    timejob=`convertsec $(($end-$begin))`  #(copy the convertsec function in my ~/.bashrc to yours)
    echo "  END >>> duration : ${timejob} `date`"

done

exit 0




for i in $(squeue | grep post$id   | awk '{print $1}') ; do scancel $i ; done
d1=2015010100 ; d2=2015093023
pdir=`pwd`
debug=1
id=1
outdir=/gpfs/scratch/pr1ehu00/pr1ehu18/data/opmos/debug/opmos_b007_MAD-RUR_2015010100_2015093023_v20190111134552
rm -f $pdir/slurm-*
rm -f $outdir/other/slurm_post/slurm-*
rm -f $outdir/post_statistics/*
    what='obs mod kfsRMSE 
ml_gbm-90-30-0-0-1-0.50 ml_gbm-90-30-0-1-1-0.50 ml_gbm-90-30-0-2-1-0.50 ml_gbm-90-30-0-3-1-0.50 ml_gbm-90-30-0-4-1-0.50 ml_gbm-90-30-0-6-1-0.50 ml_gbm-90-30-0-7-1-0.50 ml_gbm-90-30-0-11-1-0.50 
ml_gbm-90-30-0-0-1-0.75 ml_gbm-90-30-0-1-1-0.75 ml_gbm-90-30-0-2-1-0.75 ml_gbm-90-30-0-3-1-0.75 ml_gbm-90-30-0-4-1-0.75 ml_gbm-90-30-0-6-1-0.75 ml_gbm-90-30-0-7-1-0.75 ml_gbm-90-30-0-11-1-0.75 
ml_gbm-90-30-1-0-1-0.50 ml_gbm-90-30-1-1-1-0.50 ml_gbm-90-30-1-2-1-0.50 ml_gbm-90-30-1-3-1-0.50 ml_gbm-90-30-1-4-1-0.50 ml_gbm-90-30-1-6-1-0.50 ml_gbm-90-30-1-7-1-0.50 ml_gbm-90-30-1-11-1-0.50 
ml_gbm-90-30-1-0-1-0.75 ml_gbm-90-30-1-1-1-0.75 ml_gbm-90-30-1-2-1-0.75 ml_gbm-90-30-1-3-1-0.75 ml_gbm-90-30-1-4-1-0.75 ml_gbm-90-30-1-6-1-0.75 ml_gbm-90-30-1-7-1-0.75 ml_gbm-90-30-1-11-1-0.75'
pollutant='PM10' 
$pdir/top_post.sh $debug "$what" "$pollutant" $id $outdir $d1 $d2
