#!/bin/bash 

debug=$1
what="$2"
pollutant="$3"
id=$4
outdir=$5
d1=$6
d2=$7

echo "debug     : $debug"
echo "what      : $what"
echo "pollutant : $pollutant"
echo "id        : $id"
echo "outdir    : $outdir"
echo "d1        : ${d1}"
echo "d2        : ${d2}"



function waitendjob()
{
    echo -e "\e[00;31m | Check if jobs are finished... \e[00m `date`"
    ok=1
    while [ $ok -eq 0 ] ; do 
	nslurm=`ls $pdir/sl* 2> /dev/null| wc -l`
	if [ $nslurm -gt 0 ] ; then ok=1 ; fi
	sleep 1
    done
    sleep 10
    njob=999
    while [ $njob -ne 0 ] ; do
	nerr=`   grep Err    $pdir/sl* 2> /dev/null | wc -l`
	nkilled=`grep Killed $pdir/sl* 2> /dev/null | wc -l`
	ncancel=`grep CANCEL $pdir/sl* 2> /dev/null | wc -l`
	nslurm=`ls $pdir/sl*           2> /dev/null | wc -l`
	njob=`squeue | grep jpost${id} | wc -l`
	echo "waiting... ($njob still running)($nslurm slurm files)  (NERRORS/NKILLED/NCANCEL=$nerr/$nkilled/$ncancel) `date`"
	if [[ $nerr -gt 0 || $nkilled -gt 0 || $ncancel -gt 0 ]] ; then 
	    for i in $(squeue | grep jpost${id} | awk '{print $1}') ; do scancel $i ; done
	    echo "(NERRORS/NKILLED/NCANCEL=$nerr/$nkilled/$ncancel)"
	    exit 0
	fi
	sleep 10
    done
}

#############################################################################################
echo -e "\e[00;31m | ==============================================\e[00m"
echo -e "\e[00;31m | ====================  POST_MOS  ==============\e[00m"
echo -e "\e[00;31m | ==============================================\e[00m"

for i in $(squeue | grep post$id  | awk '{print $1}') ; do scancel $i ; done


echo -e "\e[00;31m | Preliminaries \e[00m `date`"
pdir=`pwd`
type='hx dx dmaxx dmda8x'
tmp=`cd $outdir ; ls data* | head -1`
#d1=${tmp:5:10}   ; echo $d1
#d2=${tmp:16:10}  ; echo $d2
rm -f $pdir/slurm*out


echo -e "\e[00;31m | Post-processed statistics... \e[00m `date`"
if [ ! -d $outdir/post_statistics ] ; then
    mkdir $outdir/post_statistics
fi


#############################################################################################
echo -e "\e[00;31m | Aggregate hopdata.Rdata and hdata.Rdata files... \e[00m `date`"
for iwhat in $what ; do		   
    for ipollutant in $pollutant ; do
	sbatch -n1 -c1 --job-name="jpost${id}A" $pdir/job_post.sh "aggregate1a" $pdir $outdir $ipollutant $iwhat $d1 $d2 &> /dev/null
	sbatch -n1 -c1 --job-name="jpost${id}A" $pdir/job_post.sh "aggregate1b" $pdir $outdir $ipollutant $iwhat $d1 $d2 &> /dev/null
    done
done
waitendjob


#############################################################################################
echo -e "\e[00;31m | Aggregate xhopdata.Rdata and xhdata.Rdata files... \e[00m `date`"
for ipollutant in $pollutant ; do
    sbatch -n1 -c1 --job-name="jpost${id}B" $pdir/job_post.sh "aggregate2a" $pdir $outdir $ipollutant "$what" $d1 $d2 &> /dev/null
    sbatch -n1 -c1 --job-name="jpost${id}B" $pdir/job_post.sh "aggregate2b" $pdir $outdir $ipollutant "$what" $d1 $d2 &> /dev/null
done
waitendjob


#############################################################################################
echo -e "\e[00;31m | Compute xxstat... \e[00m `date`"
for iwhat in $what ; do		
    if [[ "$iwhat"" != "obs" && "$iwhat"" != "mod" ]] ; then
	for ipollutant in $pollutant ; do
	    sbatch -n1 -c1 --job-name="jpost${id}C" $pdir/job_post.sh "xxstat" $pdir $outdir $ipollutant "$iwhat" $d1 $d2 &> /dev/null
	done
    fi
done


#############################################################################################
echo -e "\e[00;31m | Launch confusion/hfopstat/hx1errorbinstat/hx1obsbinstat steps... \e[00m `date`"
for iwhat in $what ; do		   
    if [[ "$iwhat" != "obs" && "$iwhat" != "mod" ]] ; then
	for ipollutant in $pollutant ; do
	    for itype in $type ; do
		sbatch -n1 -c1 --job-name="jpost${id}D" $pdir/job_post.sh "confusion"       $pdir $outdir $ipollutant $iwhat $d1 $d2 $itype &> /dev/null
	    done
	    sbatch -n1 -c1 --job-name="jpost${id}E" $pdir/job_post.sh "hfopstat"        $pdir $outdir $ipollutant $iwhat $d1 $d2 &> /dev/null
	    sbatch -n1 -c1 --job-name="jpost${id}F" $pdir/job_post.sh "hx1errorbinstat" $pdir $outdir $ipollutant $iwhat $d1 $d2 &> /dev/null
	    sbatch -n1 -c1 --job-name="jpost${id}G" $pdir/job_post.sh "hx1obsbinstat"   $pdir $outdir $ipollutant $iwhat $d1 $d2 &> /dev/null
	done
    fi
done
waitendjob


#############################################################################################
echo -e "\e[00;31m | Launch hopstat step... \e[00m `date`"
for iwhat in $what ; do		   
    if [ "$iwhat" != "obs" ] ; then
	for ipollutant in $pollutant ; do
	    sbatch -n1 -c1 --job-name="jpost${id}H" $pdir/job_post.sh "hopstat"   $pdir $outdir $ipollutant $iwhat $d1 $d2 &> /dev/null
	done
    fi
done
waitendjob


echo -e "\e[00;31m | Move slurm files ... \e[00m `date`"
mv $pdir/slurm*.out  $outdir/other/slurm_post

echo -e "\e[00;31m | END_top_post ... \e[00m `date`"



